@@ -1,5 +1,5 @@
 package Net::Async::AMQP::Channel;
-$Net::Async::AMQP::Channel::VERSION = '0.006';
+$Net::Async::AMQP::Channel::VERSION = '0.007';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Net::Async::AMQP::Channel - represents a single channel in an MQ connection
 
 =head1 VERSION
 
-Version 0.006
+Version 0.007
 
 =head1 SYNOPSIS
 
@@ -80,15 +80,15 @@ sub confirm_mode {
 
     my $f = $self->loop->new_future;
     my $frame = Net::AMQP::Frame::Method->new(
-        channel => $self->id,
+#        channel => $self->id,
         method_frame => Net::AMQP::Protocol::Confirm::Select->new(
             nowait      => 0,
         )
     );
-    $self->amqp->push_pending(
+    $self->push_pending(
         'Confirm::SelectOk' => [ $f, $self ]
     );
-    $self->amqp->send_frame($frame);
+    $self->send_frame($frame);
     return $f;
 }
 
@@ -117,7 +117,7 @@ sub exchange_declare {
 
     my $f = $self->loop->new_future;
     my $frame = Net::AMQP::Frame::Method->new(
-        channel => $self->id,
+#        channel => $self->id,
         method_frame => Net::AMQP::Protocol::Exchange::Declare->new(
             exchange    => $args{exchange},
             type        => $args{type},
@@ -129,10 +129,10 @@ sub exchange_declare {
             nowait      => 0,
         )
     );
-    $self->amqp->push_pending(
+    $self->push_pending(
         'Exchange::DeclareOk' => [ $f, $self ]
     );
-    $self->amqp->send_frame($frame);
+    $self->send_frame($frame);
     return $f;
 }
 
@@ -156,12 +156,20 @@ sub queue_declare {
     $self->future->then(sub {
         warn "queue decl\n" if DEBUG;
         my $f = $self->loop->new_future;
-        $self->add_child(my $q = Net::Async::AMQP::Queue->new(
-            amqp   => $self->amqp,
-            future => $f,
-        ));
+        $self->add_child(
+			my $q = Net::Async::AMQP::Queue->new(
+				amqp    => $self->amqp,
+				future  => $f,
+				channel => $self,
+			)
+		);
+		# Avoid the cycle caused by self -> children [ queue ],
+		# but do it outside the queue object so that we
+		# can assign a different channel elsewhere without
+		# triggering cleanup logic as soon as that channel
+		# goes out of scope.
+		Scalar::Util::weaken($q->{channel});
         warn "Attempting to declare our queue" if DEBUG;
-        $q->configure(channel => $self);
         my $frame = Net::AMQP::Frame::Method->new(
             channel => $self->id,
             method_frame => Net::AMQP::Protocol::Queue::Declare->new(
@@ -179,7 +187,7 @@ sub queue_declare {
                 nowait      => 0,
             )
         );
-        $self->amqp->push_pending(
+        $self->push_pending(
             'Queue::DeclareOk' => sub {
                 my ($amqp, $frame) = @_;
                 my $method_frame = $frame->method_frame;
@@ -242,10 +250,10 @@ sub publish {
 				weaken $f;
 				weaken $ack;
 			};
-			$self->amqp->push_pending(
+			$self->push_pending(
 				'Basic::Return' => $return,
 			);
-			$self->amqp->push_pending(
+			$self->push_pending(
 				'Basic::Ack' => $ack,
 			);
 		}
@@ -277,7 +285,7 @@ sub publish {
         );
         $self->send_frame(
             $_,
-            channel => $channel,
+#            channel => $channel,
         ) for @frames;
         $f
     })
@@ -307,12 +315,12 @@ sub qos {
     $self->future->then(sub {
         my $f = $self->loop->new_future;
         my $channel = $self->id;
-        $self->amqp->push_pending(
+        $self->push_pending(
             'Basic::QosOk' => [ $f, $self ],
         );
 
         my $frame = Net::AMQP::Frame::Method->new(
-            channel => $self->id,
+#            channel => $self->id,
             method_frame => Net::AMQP::Protocol::Basic::Qos->new(
                 nowait         => 0,
                 prefetch_count => $args{prefetch_count},
@@ -345,7 +353,7 @@ sub ack {
     $self->future->on_done(sub {
         my $channel = $id;
         my $frame = Net::AMQP::Frame::Method->new(
-            channel => $id,
+#            channel => $id,
             method_frame => Net::AMQP::Protocol::Basic::Ack->new(
                # nowait      => 0,
 				delivery_tag => $args{delivery_tag},
@@ -391,7 +399,13 @@ L<Net::Async::AMQP> instance.
 
 =cut
 
-sub send_frame { shift->amqp->send_frame(@_) }
+sub send_frame {
+	my $self = shift;
+	$self->amqp->send_frame(
+		@_,
+		channel => $self->id,
+	)
+}
 
 =head2 close
 
@@ -414,19 +428,104 @@ sub close {
 
     my $f = $self->loop->new_future;
     my $frame = Net::AMQP::Frame::Method->new(
-        channel => $self->id,
+#        channel => $self->id,
         method_frame => Net::AMQP::Protocol::Channel::Close->new(
 			reply_code  => $args{code} // 404,
 			reply_text  => $args{text} // 'closing',
         )
     );
-    $self->amqp->push_pending(
+    $self->push_pending(
         'Channel::CloseOk' => [ $f, $self ],
     );
-    $self->amqp->send_frame($frame);
+    $self->send_frame($frame);
     return $f;
 }
 
+=head2 push_pending
+
+=cut
+
+sub push_pending {
+    my $self = shift;
+    while(@_) {
+        my ($type, $code) = splice @_, 0, 2;
+        push @{$self->{pending}{$type}}, $code;
+    }
+    return $self;
+}
+
+=head2 remove_pending
+
+Removes a coderef from the pending event handler.
+
+Returns C< $self >.
+
+=cut
+
+sub remove_pending {
+	my $self = shift;
+    while(@_) {
+        my ($type, $code) = splice @_, 0, 2;
+		# This is the same as extract_by { $_ eq $code } @{$self->{pending}{$type}};,
+		# but since we'll be calling it a lot might as well do it inline:
+		splice
+			@{$self->{pending}{$type}},
+			$_,
+			1 for grep {
+				$self->{pending}{$type}[$_] eq $code
+			} reverse 0..$#{$self->{pending}{$type}};
+    }
+    return $self;
+}
+
+=head2 next_pending
+
+Retrieves the next pending handler for the given incoming frame type (see L</get_frame_type>),
+and calls it.
+
+Takes the following parameters:
+
+=over 4
+
+=item * $type - the frame type, such as 'Basic::ConnectOk'
+
+=item * $frame - the frame itself
+
+=back
+
+Returns $self.
+
+=cut
+
+sub next_pending {
+    my $self = shift;
+    my $type = shift;
+    my $frame = shift;
+    warn "Check next pending for $type\n" if DEBUG;
+    if(my $next = shift @{$self->{pending}{$type} || []}) {
+		# We have a registered handler for this frame type. This usually
+		# means that we've sent a message and are awaiting a response.
+		if(ref($next) eq 'ARRAY') {
+			my ($f, @args) = @$next;
+			$f->done(@args) unless $f->is_ready;
+		} else {
+			$next->($self, $frame, @_);
+		}
+	} else {
+		# It's quite possible we'll see unsolicited frames back from
+		# the server: these will typically be errors, connection close,
+		# or consumer cancellation if the consumer_cancel_notify
+		# option is set (RabbitMQ). We don't expect many so report
+		# them when in debug mode.
+		return undef;
+		warn "We had no pending handlers for $type, raising as event" if DEBUG;
+		$self->bus->invoke_event(
+			unexpected_frame => $type, $frame
+		);
+	}
+    $self
+}
+
 =head1 METHODS - Accessors
 
 =cut
@@ -485,7 +584,7 @@ sub as_string {
 
 {
 package Net::Async::AMQP::Channel::Helper;
-$Net::Async::AMQP::Channel::Helper::VERSION = '0.006';
+$Net::Async::AMQP::Channel::Helper::VERSION = '0.007';
 sub new { bless { channel => $_[1] }, $_[0] }
 sub channel { shift->{channel} }
 sub DESTROY {
