@@ -1,5 +1,5 @@
 package Net::Async::AMQP::ConnectionManager::Channel;
-$Net::Async::AMQP::ConnectionManager::Channel::VERSION = '0.006';
+$Net::Async::AMQP::ConnectionManager::Channel::VERSION = '0.007';
 use strict;
 use warnings;
 
@@ -9,7 +9,7 @@ Net::Async::AMQP::ConnectionManager::Channel - channel proxy object
 
 =head1 VERSION
 
-Version 0.006
+Version 0.007
 
 =cut
 
@@ -33,9 +33,61 @@ sub new {
 	my $class = shift;
 	my $self = bless { @_ }, $class;
 	Scalar::Util::weaken($_) for @{$self}{qw(manager channel)};
+#	warn "Acquiring $self\n";
+	$self->bus->subscribe_to_event(
+		my @ev = (
+			listener_start => $self->curry::weak::_listener_start,
+			listener_stop  => $self->curry::weak::_listener_stop,
+		)
+	);
+	$self->{cleanup}{events} = sub {
+		shift->bus->unsubscribe_from_event(@ev);
+		Future->wrap;
+	};
 	$self
 }
 
+sub _listener_start {
+	my ($self, $ev, $ctag) = @_;
+	$self->{cleanup}{"listener-$ctag"} = sub {
+		my $self = shift;
+#		warn "::: CLEANUP TASK - kill $ctag listener on $self\n";
+		Net::Async::AMQP::Queue->new(
+			amqp => $self->amqp,
+			channel => $self,
+			future => Future->wrap
+		)->cancel(
+			consumer_tag => $ctag
+		)
+	};
+}
+
+sub _listener_stop {
+	my ($self, $ev, $ctag) = @_;
+	# warn "No longer need to clean up $ctag listener\n" if DEBUG;
+	delete $self->{cleanup}{"listener-$ctag"};
+}
+
+=head2 queue_declare
+
+=cut
+
+sub queue_declare {
+	my ($self, %args) = @_;
+	$self->channel->queue_declare(%args)->transform(
+		done => sub {
+			my ($q) = @_;
+			# Ensure that this wrapped channel is used
+			# as the stored channel value. This means
+			# the channel holds the queue, we hold a weakref
+			# to the channel, and the queue holds a strong
+			# ref to our channel wrapper.
+			$q->configure(channel => $self);
+			$q
+		}
+	)
+}
+
 =head2 channel
 
 Returns the underlying AMQP channel.
@@ -62,7 +114,7 @@ Takes the form "Channel[N]", where N is the ID.
 
 sub as_string {
 	my $self = shift;
-	sprintf "Channel[%d]", $self->id;
+	sprintf "ManagedChannel[%d]", $self->id;
 }
 
 =head2 DESTROY
@@ -77,19 +129,23 @@ any trailing consumers, for example. These are held in the cleanup hash.
 
 sub DESTROY {
 	my $self = shift;
-	my $conman = delete $self->{manager};
-	my $ch = delete $self->{channel};
-	$conman->debug_printf("Releasing channel %d", $ch->id);
-	return $conman->release_channel($ch) unless $self->{cleanup};
+	unless($self->{cleanup}) {
+#		warn "Releasing $self without cleanup\n";
+		my $conman = delete $self->{manager};
+		return $conman->release_channel(delete $self->{channel});
+	}
+
+#	warn "Releasing $self\n";
 	my $f;
 	$f = (
 		fmap_void {
-			$_->()
+			$_->($self)
 		} foreach => [
 			sort values %{$self->{cleanup}}
 		]
 	)->on_ready(sub {
-		$conman->release_channel($ch);
+		my $conman = delete $self->{manager};
+		$conman->release_channel(delete $self->{channel});
 		undef $f;
 	});
 }
